@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2014,2015,2017,2018,2020
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2014-2024
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -26,6 +26,7 @@
   #:use-module (artanis route)
   #:use-module (artanis page)
   #:use-module (artanis irregex)
+  #:use-module (artanis security nss)
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
   #:use-module (web request)
@@ -152,7 +153,7 @@
          `(private ,(cons 'max-age m))))
       (else (throw 'artanis-err try-to-cache-dynamic-content
                    "Invalid opts `~a'!" o))))
-  (cache-to-tlb! rc etag) ; cache the hash the TLB
+  (cache-to-tlb! rc etag) ; cache the hash to the TLB
   (response-emit body #:headers `(,@headers
                                   (ETag . ,etag)
                                   (cache-control . ,(->cc cache-opts)))))
