@@ -1,6 +1,6 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013-2023
-;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
+;;  Copyright (C) 2013-2025
+;;      "Mu Lei" known as "NalaGinrut" <mulei@gnu.org>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
 ;;  Lesser General Public License published by the Free Software
@@ -229,8 +229,20 @@ port. When requests come, the Linux kernel will do the necessary lock and alloca
 work for us to dispatch requests to these Artanis instances.
 server.multi = <boolean>")
 
+    ((server workers)
+     1
+     "To specify the number of workers in the server core. Each worker implies a coroutine queue.
+The default value is 1, which means no threading at all.
+The throughput will be dramatically increased when you set it to a larger number.
+[Experimental] It's not well tested yet, but it does work. Here're known issues:
+1. The main thread may be halted when ctrl+c is pressed, but it still work with other threads.
+2. There could be potential race conditions, we've already use parameters for its functional nature
+   to implement thread local storage. But need more tests.
+NOTE: If the workers is larger than 1, then it implies server.multi = true.
+server.workers = <integer>")
+
     ((server websocket)
-     #t
+     #f
      "Enable WebSocket.
 server.websocket = <boolean>")
 
@@ -293,7 +305,7 @@ websocket.timeout = <integer>")
     ((host name)
      #f
      "If disabled, you will have to use the IP address to connect instead of the
-hostname. e.g. `host.name = lambdachip.com'.
+hostname. e.g. `host.name = artanis.dev'.
 host.name = <string> | disable")
 
     ((host addr)
@@ -337,54 +349,62 @@ file: stores session information into text files.
 redis: uses Redis for managing sessions.
 session.engine = simple | db | file | redis | redis@addr:port | <third-party-engine>")
 
+    ((session i18n)
+     json
+     "Specify the i18n engine. Here're supported backends:
+1. json: uses json file for i18n.
+2. sxml: uses SXML for i18n.
+3. locale: uses PO with gettext for i18n.
+session.i18n = json | sxml | locale | <third-party-engine>")
+
     ((upload types)
      (jpg png gif)
      "Specify allowed upload file type, say, upload.types = jpg,png,gif.
-upload.types = <item-list>")
+     upload.types = <item-list>")
 
     ((upload path)
      "upload"
      "The path to put the uploaded files.
-upload.path = <PATH>")
+     upload.path = <PATH>")
 
     ((upload size)
      5242880  ; 5MB
      "The size limitation of uploaded file in bytes.
-upload.size = <interger>")
+     upload.size = <interger>")
 
     ;; for mail namespace
     ((mail sender) "/usr/bin/msmtp"
      "The command called by Sendmail module. It's strongly recommended to use `msmtp' to replace `sendmail'.
-mail.sender = <string>")
+     mail.sender = <string>")
 
     ((cache maxage)
      3600
      "The maximum age of a cached page in seconds.
-This is the global maxage of any cache.
-If you want to specify maxage for certain page, please read the manual about the Cache.
-cache.maxage = <integer>")
+     This is the global maxage of any cache.
+     If you want to specify maxage for certain page, please read the manual about the Cache.
+     cache.maxage = <integer>")
 
     ((debug enable)
      #f
      "Wheather to enable debug mode.
-If you enable debug mode, Artanis will print debug information verbosely.
-The module you modified will be reloaded instantly, and the page view will be rendered as well.
-NOTE: This option will affect the performance, please use it for debug purposes only./
-debug.enable = <boolean>")
+     If you enable debug mode, Artanis will print debug information verbosely.
+     The module you modified will be reloaded instantly, and the page view will be rendered as well.
+     NOTE: This option will affect the performance, please use it for debug purposes only./
+     debug.enable = <boolean>")
 
     ((debug monitor)
      ()
      "The paths that needs to be monitored in debug-mode.
-This will take advantage of `inotify' in GNU/Linux kernel.
-NOTE: We may support GNU/Hurd as well, with its file monitor mechanism, in the future.
-debug.monitor = <PATHs>")
+     This will take advantage of `inotify' in GNU/Linux kernel.
+     NOTE: We may support GNU/Hurd as well, with its file monitor mechanism, in the future.
+     debug.monitor = <PATHs>")
     ((cookie expires)
      3600
      "Cookie expiration time in seconds.
-       1 hour is 3600
-       6 hours 21600
-       1 month 2592000
- cookie.expires = <integer>")))
+     1 hour is 3600
+     6 hours 21600
+     1 month 2592000
+     cookie.expires = <integer>")))
 
 ;; Init all fields with default values
 (for-each (lambda (x) (conf-set! (car x) (cadr x))) (default-conf-values))
@@ -464,6 +484,12 @@ debug.monitor = <PATHs>")
                 #f
                 "Invalid db.pool value, we accept: ~{~a~^,~}" *pool-modes*)))))
 
+(define-syntax-rule (->workers x)
+  (let ((w (string->number x)))
+    (cond
+     ((positive? w) w)
+     (else (error "Invalid server.workers number! Must be >= 1" x)))))
+
 (define (parse-namespace-db item)
   (match item
     (('enable usedb) (conf-set! '(db enable) (->bool usedb)))
@@ -478,7 +504,7 @@ debug.monitor = <PATHs>")
     (('poolsize poolsize) (conf-set! '(db poolsize) (->integer poolsize)))
     (('pool pool) (conf-set! '(db pool) (->pool-mode pool)))
     (('encodeparams encodeparams) (conf-set! '(db encodeparams) (->bool encodeparams)))
-    (('lpc lpc) (conf-set! '(db lpc) lpc) (->bool lpc))
+    (('lpc lpc) (conf-set! '(db lpc) (->bool lpc)))
     (else (error parse-namespace-db "Config: Invalid item" item))))
 
 (define (parse-namespace-server item)
@@ -495,6 +521,7 @@ debug.monitor = <PATHs>")
     (('bufsize bufsize) (conf-set! '(server bufsize) (->integer bufsize)))
     (('impl impl) (conf-set! '(server impl) (string->symbol impl)))
     (('multi multi) (conf-set! '(server multi) (->bool multi)))
+    (('workers workers) (conf-set! '(server workers) (->workers workers)))
     (('engine engine) (conf-set! '(server engine) (->symbol engine)))
     (('websocket websocket) (conf-set! '(server websocket) (->bool websocket)))
     (('pub pub) (conf-set! '(server pub) (basename pub)))
@@ -540,6 +567,7 @@ debug.monitor = <PATHs>")
     (('path path) (conf-set! '(session path) path))
     (('hijackcheck check) (conf-set! '(session hijackcheck) (->bool check)))
     (('backend backend) (conf-set! '(session backend) (parse-namespace-backend backend)))
+    (('i18n i18n) (conf-set! '(session i18n) (->symbol i18n)))
     (else (error parse-namespace-session "Config: Invalid item" item))))
 
 (define (parse-namespace-upload item)
@@ -570,6 +598,11 @@ debug.monitor = <PATHs>")
     (('expires expires) (conf-set! '(cookie expires) (->integer expires)))
     (else (error parse-namespace-cookie "Config: Invalid item" item))))
 
+(define (parse-custom-conf item)
+  (match item
+    ((key value) (conf-set! `(custom ,key) value))
+    (else (error parse-custom-conf "Config: Invalid item" item))))
+
 (define (parse-config-item item)
   (match item
     (('db rest ...) (parse-namespace-db rest))
@@ -582,7 +615,8 @@ debug.monitor = <PATHs>")
     (('cache rest ...) (parse-namespace-cache rest))
     (('debug rest ...) (parse-namespace-debug rest))
     (('cookie rest ...) (parse-namespace-cookie rest))
-    (((? string-null?)) #t) ; skip
+    (('custom rest ...) (parse-custom-conf rest))
+    (((? string-null?)) #t)             ; skip
     (else (error parse-config-item "Unsupported config namespace!" item))))
 
 (define (parse-line line)
